//---------------------------------------------------------------------------
#pragma hdrstop

#include "Machine.h"
#include "VacuumGauge.h"
#include "Utility.h"
#include <math.h>
#include "MachinePropertiesForm.h"
#include "MachineFlgFrm.h"
#include "ViewMachineErrors.h"
#include "MachineStatusForm.h"
#include "XMLUtilities.h"
#include "XLSimulatorFrm.h"
#include "AOSTestClient.h"
#include "ViewMachineXML.h"

TMachine * Machine;
//---------------------------------------------------------------------------
__fastcall TMachine::TMachine(TComponent *Owner):XLCommIn(Owner)
{///Constructor
	XLActual->putTempInt(-1);                       ///Set the actual temp to a bogus value so that we don't get a DifTemp = 0 on program startup
	bRotorSpinning=false;                           ///bRotorSpinning set true if rpm>0
	bStopping=false;                                ///Set to false unless the 'Stop machine' option is selected or machine state is 'Stopping'... this takes care of a problem with the Start/Stop buttons
	bMachineStarted=false;                          ///If the machine has been told to start, this logical is set
	XLSocket = new TAOSMachineSocket(this);					/// Create the server socket to connet to remote clients
	ReadPropertiesDB();															/// Get the name, serial number, etc. for this instrument
	XLAAbsorbanceAvailable = false;									/// XLA absorbance software not in place, yet
 ///Set up the update timer
	UpdateTimer = new TTimer(this);                 /// Main loop timer to update the machine status
	UpdateTimer->Enabled=false;                     /// We will enable this in a bit
	UpdateTimer->Interval=2500;  										/// Initial update period = 2.5 seconds
	UpdateTimer->OnTimer=UpdateTime;
 ///Set up timer to see if the machine has connected
	ConnectTimer = new TTimer(this);                /// Timer used to determine if communications have been established
	ConnectTimer->Enabled=false;                    /// We will enable this in a bit
	ConnectTimer->Interval=500;  										/// Initial update period = 0.5 seconds
	ConnectTimer->OnTimer=ConnectTime;							/// Function called when update timer fires
	ConnectionStatus = "Not connected";						  /// This will be updated as connection is attempted
	bInitializingProgram=true;                    	/// Set to true (default on program start) until communications with XL established, steppers are initialized
 ///Machine status form
	MachineStatus   = new TMachineStatusFrm(this);  /// The form that displays the machine's status
	TBorderIcons tBI = MachineStatus->BorderIcons;  /// This is a bare-ass form
	tBI >> biMaximize;
	tBI >> biMinimize;
	tBI >> biSystemMenu;
	MachineStatus->BorderIcons = tBI;
	Simulator = new TXLSimFrm(this);
}
//---------------------------------------------------------------------------
__fastcall TMachine::~TMachine()
{///Destructor
	WritePropertiesDB();
	if(Util->FormExists("MachineFlagFrm")){
		delete MachineFlagFrm;																										///Have one of these, so delete it
	}
	if(Util->FormExists("MachineStatusFrm")){																		///Have one of these, so delete it
		delete MachineStatus;
	}
	if(Util->FormExists("XLSimFrm")){																						///Have one of these, so delete it
		delete Simulator;
	}
	if(Util->FormExists("MachinePropFrm")){																			///Have one of these, so delete it
		delete MachinePropFrm;
	}
	if(Util->FormExists("CommSet")){																						///Have one of these, so delete it
		delete XLCommSettings;
	}
	if(Util->FormExists("ViewMachineErrorFrm")){																///Have one of these, so delete it
		delete ViewMachineErrorFrm;
	}
	if(Util->FormExists("MachineXMLViewer")){																		///Have one of these, so delete it
		delete MachineXMLViewer;
	}
	if(XLSocket)					delete XLSocket;
}
//---------------------------------------------------------------------------
bool __fastcall TMachine::StartMachine()
{///Sequence of steps needed to issue a STARTFUGE command to the XLI.
	bool ret=false;
	CheckDesiredSettings();                      					/// Give the machine the desired settings
	Sleep(100);                                           /// Make sure the CheckDesired settings digested before sending a STARTFUGE command
	if(!bMachineStarted)
		{///Either the machine or the experiment is being started. A start command needs to be issued
			SendXLMessage(STARTFUGE);                					/// Start the centrifuge
			bStopping=false;
			bMachineStarted=true;
			MachineStartTime=(double)Util->GetTimeSeconds();
			//-->RunLog->AddLog(23);                          /// Log machine starting
			if(bSimulation){
			 Simulator->StartMachine();      									/// Issue the sequence of commands that simulates starting the machine
			}
			ret = true;
		}
	MachineStatus->UpdateDisplay();												/// Display the most recent settings- This will show the latest status
	return ret;
}
//---------------------------------------------------------------------------
bool __fastcall TMachine::StopMachine()
{///Sequence of steps needed to issue a STARTFUGE command to the XLI.
	SendXLMessage(STOPFUGE);
	Sleep(100);
	RequestedRotorSpeed = "0";
	bStopping=true;             																		/// Let the program know what the state of the machine is
	bMachineStarted=false;
	//-->RunLog->AddLog(23);                                        ///	Log rotor stopping
	if(bSimulation){                                                /// Simulate stop actions if this is a simulation
	 Simulator->StopMachine();
	}
	MachineStatus->UpdateDisplay();																	/// Display the most recent settings- This will show the latest status
	return true;
}
//---------------------------------------------------------------------------
void TMachine::StartPumping()
{///Sets the machine to 0 rpm and begins timing how long it takes to get to 50 microns
	if(VacuumSystem->bPumpDown) return;                     			///	Already started
	AnsiString s = RequestedRotorSpeed;														/// Want to keep this value and reset the rpm to it when the vacuum is OK
	SetRotorSpeed(0);                                       			///	Start the 'spinning' at 0 rpm so that the diffusion pump fires up
//-->  Experiment->RunLog->AddLog(31);                    			/// Log the pump starting
	RequestedRotorSpeed = s;																			/// When the vacuum is OK, we'll reset the rotor speed to this value
	VacuumSystem->StartPumping();																	/// Set the flags and start pumping time
	TemperatureSystem->bTemperatureOK=false;                			/// No way the temp could be correct... have to be below 100 microns
	if(bSimulation){///Simulate what happens when the pumps are started
		Simulator->StartPumping();
	}
}
//---------------------------------------------------------------------------
void TMachine::CloseCommPort()
{/// Closes the comport, sets the connection status to not connected
	CloseDevice();
	XLActual->ActualsObtained(false);
	XLDesired->SettingsObtained(false);
	ConnectionStatus = "Not connected";						  		/// This will be updated as connection is attempted
	ConnectTimer->Enabled = false;											/// Start timing the connection
	MachineStatus->UpdateDisplay();												/// Display the most recent settings- This will show the latest status
}
//---------------------------------------------------------------------------
bool TMachine::InitializeCommPort()
{///Opens the commport, sends the calparms and requests settings, returns true if successful, and port open
	t1=(double)Util->GetTimeSeconds();									/// Set the first update time to now
	if(IsOpen){
	 CloseDevice();                 										/// If the ComPort is already open, close it down nicely
	}
	if(OpenDevice()){																		/// Open the commport and set IsOpen
		SendXLMessage("Z");                       		 		/// Send cal file at startup
		SendXLMessage(REQUESTSETTINGS);           				/// Request current settings
		ConnectTimer->Enabled = true;											/// Start timing the connection
		ConnectTime(this);																/// And update right away
	}
	else{																								/// Failed to open the port handle
		ConnectTimer->Enabled = false;
		ConnectionStatus = Port.UpperCase()+" Failed to open";
	}
	MachineStatus->UpdateDisplay();												/// Display the most recent settings- This will show the latest status
	return IsOpen;
}
//---------------------------------------------------------------------------
void TMachine::SetupForSimulation()
{/// Sets the bSimulation flag true, initializes the XLActual/Desired arrays
	TemperatureSystem->RequestedTemperature = "20"; 		/// If testing, set the temperature
	RequestedRotorSpeed = "0";														 /// Set the desired speed to 0
	XLDesired->putTemp("200");													 /// Set the desired temperature to 20.0
	XLActual->putTemp("200");                            /// Set the actual temperature to 20.0
	SetDesiredAcceleration("400");											 /// Set the accel/decel to defaults
	SetDesiredDeceleration("400");                       /// Set the default
	XLActual->putAnDecel(XLDesired->getAnDecel());
	XLActual->putAnAccel(XLDesired->getAnAccel());
	XLActual->putSpeed(RequestedRotorSpeed.c_str());       /// Fill the structure that is used to xmit to XLA
	XLDesired->putSpeed(RequestedRotorSpeed.c_str());      /// Fill the structure that is used to xmit to XLA
	AnsiString hold="hold";
	XLDesired->putTime(hold.c_str());           				 /// Make sure the machine is in Hold mode
	XLActual->putTime(hold.c_str());            				 /// Make sure the machine is in Hold mode
	bSimulation = true;                                  /// Set the flag that we are simulating the XL
	MachineStatus->UpdateDisplay();
}
//---------------------------------------------------------------------------
void __fastcall TMachine::ShowSimulator()
{/// Make the simulator form visible
	Simulator->Show();
}
//---------------------------------------------------------------------------
void __fastcall TMachine::SetRotorSpeed(int DesiredSpeed)
{///Set the rotor speed to value in DesiredSpeed
	RequestedRotorSpeed = (AnsiString)DesiredSpeed;      									/// This is the correct desired speed
	CheckDesiredSettings();                          									/// If this entails a change in rpm, then make sure the XL gets the message
	if(bSimulation){
		Simulator->TargetRpm=DesiredSpeed;
		UpdatePumping();
	}
 ///Log changes
	if((DesiredSpeed > 0) && !bRotorSpinning)
		{///Log acceleration
	//-->			Experiment->RunLog->AddLog(21);
			StartSpinningTime=Util->GetTimeSeconds();                     /// Time in seconds when the machine first started spinning
		}
	//	if((DesiredSpeed==0 && bRotorSpinning))
	//-->		Experiment->RunLog->AddLog(23); ///Log stopping
}
//---------------------------------------------------------------------------
int TMachine::RpmChange()
{///Returns the change in rpm from most recent to the last rpm
	int Rpm = XLActual->getSpeedInt();
	return (Rpm-LastRpm);
}
//---------------------------------------------------------------------------
bool TMachine::IsRotorSpeedStable(int RPMDiff)
{///Returns true if the change rotor speed between the last and the most recent rpm measurements is less than RPMDiff.
	bool ret=true;
	int Rpm = XLActual->getSpeedInt();
	int rpmdiff = abs(LastRpm-Rpm); ///Will test on the change in rpm since the last update
	if(rpmdiff>RPMDiff) ret=false;
	return ret;
}
//---------------------------------------------------------------------------
bool TMachine::IsAtSpeed(int RPMdes, int RPMDiff)
{
	bool ret=true;
	int Rpm = XLActual->getSpeedInt();
	int rpmdiff = abs(RPMdes-Rpm); 																		///Will test on the change in rpm since the last update
	if(rpmdiff>RPMDiff) ret=false;
	return ret;
}
//---------------------------------------------------------------------------
bool TMachine::CheckForStableState(int TRpm)
{///Test whether or not machine is in stable state
 return (TemperatureSystem->bTemperatureOK   												///Is temperature OK?
		&& IsAtSpeed(TRpm,100));            														///Have we reached the correct rotor speed
}
//---------------------------------------------------------------------------
void TMachine::CheckDesiredSettings()
{///Check whether the DesiredRotorSpeed and DesiredTemperature match those on the XL. Set the XL to the Desired values
	if(RequestedRotorSpeed != GetDesiredRotorSpeedAsString()){									///The desired rotor speed in the program does not match the XL desired settings... make the XL settings match what the program wants
		XLDesired->putSpeed(RequestedRotorSpeed.c_str());       										///Fill the structure that is used to xmit to XLA
		XLDesired->Changed = true;
		if(bSimulation){                                                          ///Simulate changes that need to be made
			Simulator->CheckDesiredSettings();
		}
	}
	if(TemperatureSystem->RequestedTemperature != GetDesiredTemperatureAsString()){///The desired temperature in the program does not match the XL desired settings... make the XL settings match what the program wants
		XLDesired->putTemp(TemperatureSystem->RequestedTemperature.c_str());
		XLDesired->Changed=true;
	}
	if(GetDesiredAccelerationAsInt() != GetActualAccelerationAsInt()){
		XLDesired->Changed=true;
	}
	if(GetDesiredDecelerationAsInt() != GetActualDecelerationAsInt()){
		XLDesired->Changed=true;
	}
	if(XLDesired->Changed){
		SendXLMessage(SENDSETTINGS); 												///If there is a difference, then make sure the change is transmitted to the XL
		XLDesired->Changed=false;
		Sleep(100);                  												///Give the XLI a chance to digest these wishes
	}
	MachineStatus->UpdateDisplay();												/// Display the most recent settings- This will show the latest status
}
//---------------------------------------------------------------------------
AnsiString TMachine::GetSysStatus()
{///Returns an AnsiString with the Machine status, rpm, and temperature for adding to the notes
	AnsiString s;
	s = "AOS Machine socket: " + ConnectionState();
	s = s +" "+GetMachineStatus();
	s = s +" Rpm: "+ GetActualRotorSpeedAsString();
	s = s +	" Temp: " + GetActualTemperatureAsString();
	return s;
}
//===========================================================================
// Update functions
//---------------------------------------------------------------------------
void TMachine::UpdateStartStatus()
{///Make sure the machine is started if vac ok and StartMachine issued and not spinning
	if(bMachineStarted && VacuumSystem->bVacuumOK && !bRotorSpinning
			&& !IsAtSpeed(GetDesiredRotorSpeed(),100)){										/// Need to send settings and then issue start command again.
		//bStopping=false;
		CheckDesiredSettings();      																		/// Make sure the desired settings are set at the machine correctly
	}
}
//---------------------------------------------------------------------------
void   TMachine::UpdatePumping()
{///Test the status of pumping, set flags and update log according to the current pressure
	if(XLActual->getVacInt() < 1000
		&& !VacuumSystem->bVacuumOK){																	/// Has the machine sent over a 'V' indicating the pressure is < 50 microns.
		VacuumSystem->bVacuumOK = true;																/// Flag that the system is at vacuum
		CheckDesiredSettings();																				/// We've been pumping, so rpm = 0. Check to see if the requested speed is 0
	}
	if(bSimulation){																								/// Allow simulation of pumping... update the simulated pressure
		VacuumSystem->SimulatePumping();															/// Simulates a pumping time
		Simulator->bOKToAccelerate = VacuumSystem->bVacuumOK;					/// Allow the machine to accelerate if the vacuum is OK
		if(VacuumSystem->bVacuumOK){
			XLActual->putVac("50");
		}
	}
}
//---------------------------------------------------------------------------
void __fastcall TMachine::ResetW2t()
{/// Reset the parameters used to calculate W2t
	if(bRotorSpinning && !bStopping){ 															/// The machine is running, get w2t from the XLActuals
		w2t = GetW2tAsDouble();
	}
	else{
		w2t = 0;
		LastUpdateTime = Util->GetTimeSeconds();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMachine::UpdateW2t()
{/// Calculate W2t based on the last time it was updated, and the rpm- NOTE: If the machine is already running, then W2t needs to be pulled from the machine
	double deltat=(double)Util->GetTimeSeconds()-LastUpdateTime;
	w2t=w2t+deltat*Util->RpmToW2(Machine->GetActualRotorSpeed());
	AnsiString s=s.FormatFloat("0.0000E-00",w2t);
	XLActual->SaveW2t(s);
	LastUpdateTime = Util->GetTimeSeconds();
}
//---------------------------------------------------------------------------
void TMachine::UpdateTemperature()
{//Update the temperature status
	if(!TemperatureSystem->Update(GetDesiredTemperatureAsString())){					///Log error
		MachineDB->AddError(961,"TMachine::UpdateTemperature failed");					///Use default for severity & action
	}
}
//---------------------------------------------------------------------------
void __fastcall TMachine::SetTemperature(int temp)
{/// Set the desired temperature and update the settings
	TemperatureSystem->RequestedTemperature = (AnsiString)temp;				      												/// This is the correct desired speed
	CheckDesiredSettings();                          													/// If this entails a change in rpm, then make sure the XL gets the message
 ///Log changes
	//-->			Experiment->RunLog->AddLog(21);
}
//===========================================================================
///Acceleration/deceleration functions
//---------------------------------------------------------------------------
int  __fastcall TMachine::GetDesiredAccelerationAsInt()
{/// Get the AnAcceleration
	int i;
	i = atof(XLDesired->getAnAccel());
	return i;
}
//---------------------------------------------------------------------------
int  __fastcall TMachine::GetDesiredDecelerationAsInt()
{/// Get the AnDeceleration
	int i;
	i = atof(XLDesired->getAnDecel());
	return i;
}
//---------------------------------------------------------------------------
AnsiString  __fastcall TMachine::GetDesiredAccelerationAsString()
{/// Get the AnAcceleration
	return (AnsiString)GetDesiredAccelerationAsInt();
}
//---------------------------------------------------------------------------
AnsiString  __fastcall TMachine::GetDesiredDecelerationAsString()
{/// Get the AnDeceleration
	return (AnsiString)GetDesiredDecelerationAsInt();
}
//---------------------------------------------------------------------------
void __fastcall TMachine::SetDesiredAcceleration(AnsiString i)
{/// Set the desired acceleration
	XLDesired->putAnAccel(i.c_str());
	CheckDesiredSettings();
}
//---------------------------------------------------------------------------
void __fastcall TMachine::SetDesiredDeceleration(AnsiString i)
{/// Set the desired deceleration
	XLDesired->putAnDecel(i.c_str());
	CheckDesiredSettings();
}
//---------------------------------------------------------------------------
int  __fastcall TMachine::GetActualAccelerationAsInt()
{/// Get the AnAcceleration
	int i;
	i = atof(XLActual->getAnDecel());
	return i;
}
//---------------------------------------------------------------------------
int  __fastcall TMachine::GetActualDecelerationAsInt()
{/// Get the AnDeceleration
	int i;
	i = atof(XLActual->getAnDecel());
	return i;
}
//---------------------------------------------------------------------------
AnsiString  __fastcall TMachine::GetActualAccelerationAsString()
{/// Get the AnAcceleration
	return (AnsiString)GetActualAccelerationAsInt();
}
//---------------------------------------------------------------------------
AnsiString  __fastcall TMachine::GetActualDecelerationAsString()
{/// Get the AnDeceleration
	return (AnsiString)GetActualDecelerationAsInt();
}
//===========================================================================
// Machine status and state
//---------------------------------------------------------------------------
AnsiString TMachine::GetMachineStatus()
{ //Returns a string describing the running status of the XL
	/* defined XL statuses from fxldata.h
		#define	XLRUNNING			R     <---This is what Beckman sends when the rotor isn't spinning!
		#define	STOPPED   		S     <--- This is what Beckman sends when the rotor is spinning
		#define ACCELERATING	A
		#define DECELERATING	D
		#define	VAC_WAIT  		true/false  <-- from voltage on
	*/
	if(bSimulation){																														///Fake the status if simulating the XL
		return Simulator->GetMachineStatus();
	}
	if (CommunicationsEstablished()){																						///Actual settings
			switch (XLActual->getXlStatus())
			{///Note: WE DIDN'T SELECT THESE STATUS MESSAGES- E.g. STOPPING=RUNNING. Beckman did this long ago and far away
				case XLRUNNING:
					return "Power on";
				case STOPPING:
					return "Running";
				case ACCELERATING:
					return "Accelerating";
				case DECELERATING:
					return "Decelerating";
				default:
					return "Unknown";
			}
	}
	else return "Not connected";
}
//--------------------------------------------------------------------------
AnsiString TMachine::CurrentMachineState()
{///Returns a string that says "Waiting for: " and adds vacuum, temperature or rotor speed depending on how things are going
	AnsiString s;
	bool appended=false;
	s="Waiting for: ";
	if(!VacuumSystem->bVacuumOK)
		{///Vacuum not ready, so add that to the string
			s=s+"vacuum";
			appended=true;
		}
	if(!TemperatureSystem->bTemperatureOK)
		{///Temperature not ready, so add that to the string
			if(appended) s=s+" &";
			s=s+" temperature";
			appended=true;
		}
	if(!IsAtSpeed(GetDesiredRotorSpeed(),100))
		{///Rotor speed not ready, so add that to the string
			if(appended) s=s+" &";
			s=s+" rotor speed";
		}
	s=s+".";
	return s;
}
//---------------------------------------------------------------------------
bool __fastcall TMachine::CommunicationsEstablished()
{///Returns true if the XLActual and Desired settings have been obtained
	return (XLActual->ActualsObtained() && XLDesired->SettingsObtained());
}
//===========================================================================
// Database functions
//---------------------------------------------------------------------------
bool __fastcall TMachine::ReadPropertiesDB()
{/// Read the properties from the DB
	bool ret = true;
	if(!MachineDB->OpenMachineProperties()){
		return false;
	}
	try{
		MachineDB->MachineProperties->First();
		Port = MachineDB->MachineProperties->FieldByName("COMPort")->AsString;
		SerialNumber = MachineDB->MachineProperties->FieldByName("SerialNumber")->AsString;
		Name = MachineDB->MachineProperties->FieldByName("Name")->AsString;
		GUID = MachineDB->MachineProperties->FieldByName("GUID")->AsString;
		if(GUID == ""){
			XMLUtilities * xu;
			xu = new XMLUtilities(this);
			GUID = xu->NewGUIDString();
			delete xu;
			WritePropertiesDB();
		}
	}
	catch(...){
		ret = false;
	}
	return ret;
}
//---------------------------------------------------------------------------
bool __fastcall TMachine::WritePropertiesDB()
{/// Save the DB information
	bool ret = true;
	if(!MachineDB->OpenMachineProperties()){
		return false;
	}
	try{
		MachineDB->MachineProperties->First();
		MachineDB->MachineProperties->Edit();
		MachineDB->MachineProperties->FieldByName("COMPort")->AsString = Port;
		MachineDB->MachineProperties->FieldByName("SerialNumber")->AsString = SerialNumber;
		MachineDB->MachineProperties->FieldByName("Name")->AsString = Name;
		MachineDB->MachineProperties->FieldByName("GUID")->AsString = GUID;
		MachineDB->MachineProperties->Post();
		MachineDB->MachineProperties->MergeChangeLog();
	}
	catch(...){
  	ret = false;
	}
	return ret;
}
//===========================================================================
// Forms that can be used locally (by the server software) 
//---------------------------------------------------------------------------
void __fastcall TMachine::EditPropertiesDB()
{/// Open the Properties db for editing
	if(Util->CanShowForm(MachinePropFrm)){																			/// If one exists, bring it forward
		return;
	}
	MachinePropFrm = new TMachinePropFrm(this);                                 /// Otherwise create a new one
	MachinePropFrm->Show();
}
//---------------------------------------------------------------------------
void __fastcall TMachine::ShowCommPort()
{/// Shows the commport settings, allows commands to be sent and received
	if(Util->CanShowForm(CommSet)){																						/// If one exists, bring it forward
		return;
	}
	XLCommSettings = new TCommSet(this);
	XLCommSettings->Show();
}
//---------------------------------------------------------------------------
void __fastcall TMachine::SetCommPort(AnsiString s)
{/// Set the commport, make sure it is in the db
	if(MachineDB->ChangeProperty("COMPort",s)){
		ReadPropertiesDB();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMachine::ShowMachineFlags()
{/// Show the status flags
	if(Util->CanShowForm(MachineFlagFrm)){																			///Already have one of these, so just bring forward
		return;
	}
	MachineFlagFrm = new TMachineFlagFrm(this);
	MachineFlagFrm->Show();
}
//---------------------------------------------------------------------------
void __fastcall TMachine::ShowErrorLog()
{///View the error form
	if(Util->CanShowForm(ViewMachineErrorFrm)){																	/// If one exists, bring it forward
		return;
	}
	ViewMachineErrorFrm = new TViewMachineErrorFrm(this);
	ViewMachineErrorFrm->Show();
}
//---------------------------------------------------------------------------
void __fastcall TMachine::ShowMachineXML()
{/// Displays form that allows the XMLRPC for the instrument to be viewed
	if(Util->CanShowForm(MachineXMLViewer)){																		/// If one exists, bring it forward
		MachineXMLViewer->BringToFront();
		return;
	}
	MachineXMLViewer = new TMachineXMLViewer(this);
	MachineXMLViewer->Show();
}
//===========================================================================
// Timer functions
//---------------------------------------------------------------------------
void __fastcall TMachine::UpdateTime(TObject *Sender)
{/// Function called when update timer fires
	UpdateZeroingSpeed();                                 /// Check whether the machine has reached 0 rpm
	UpdateStartStatus();                         					/// Make sure the machine is started if vac ok and StartMachine issued and not spinning
	UpdateRotorSpeed();																		/// Update the bSpinning flag
	UpdateMachineStatus();																/// Sends REQUESTSETTINGS & REQUESTSTATUS at proper time
	UpdatePumping();																			/// Updates the vacuum status
	UpdateTemperature();																	/// Update the temperature system
	UpdateW2t();																					/// Update the calculation of w2t
	if(MachineError){																			/// There's been a serious error causing the machine to shut down
		//--> Broadcast serious error, machine shutdown
	}
	if(IsSimulation()&& bRotorSpinning){									///Need to fill the XLActual array with the current values of elapsed time and w^2t
		Simulator->UpdateTime();
	}
	UpdateStartStop();																		/// This will update the CanStart/CanStop flags
	MachineStatus->UpdateDisplay();												/// Display the most recent settings- This will show the latest status
}
//---------------------------------------------------------------------------
void __fastcall TMachine::ConnectTime(TObject *Sender)
{///Check to see if the XLActuals and XLDesired have been set. Falls through here every 500 ms
	static int RequestSettings = 0;
	double t2=(double)Util->GetTimeSeconds()-t1;
	ConnectionStatus = Port.UpperCase()+" trying to connect: ";
	ConnectionStatus = ConnectionStatus+Util->IntToHMS(t2);											/// Update the connection status message
	if(RequestSettings == 20){																									/// Will request the desired settings every 10 seconds
		SendXLMessage(REQUESTSETTINGS);                                  					/// Send the request
		RequestSettings = 0;                                                      /// Reset the interval
		MachineStatus->UpdateDisplay();																						/// Update the display
		if(bSimulation){                  																				/// Will simulate connections being made
			XLDesired->SettingsObtained(true);																			/// Set the desired having been obtained as true
			XLActual->ActualsObtained(true);                                        /// and the actuals having been obtained as true
		}
	}
	if(RequestSettings==5
		|| RequestSettings == 10
		|| RequestSettings == 15){																								///Will request the status every 2.5 seconds
		SendXLMessage(REQUESTSTATUS);	 																						/// Request the current machine settings if change has occurred
		MachineStatus->UpdateDisplay();                                           /// Update the display
	}
	++RequestSettings;
	if(CommunicationsEstablished()){																						/// If the machine and computer are talking, then set up status flags
			ConnectTimer->Enabled = false;																					/// No longer try to connect
			RequestedRotorSpeed = GetDesiredRotorSpeedAsString();  									/// Set the desired rotor speed to the current speed
			TemperatureSystem->RequestedTemperature = GetDesiredTemperatureAsString();/// Using the value from the machine
			if(XLActual->getSpeedInt() != 0){																				/// The rotor is turning
				if(GetMachineStatus() == "Decelerating"){															/// We are guessing here that the rotor is turning, but the machine has been asked to stop
					bStopping = true;
				}
				else{                                                                 /// For all other conditions, assume that it is not stopping
					bStopping = false;																									/// Set the bStopping flag
				}
				bMachineStarted=true;
				AnsiString stime = XLDesired->getAnsiTime();													/// Determine whether the XLA is in hold mode, or on the timer
				if(stime =="Hold"){																										/// The time in XLActual is the elapsed time
					StartSpinningTime = Util->GetTimeSeconds() - GetActualTimeInSeconds();
				}
				else{																																	/// XLActual holds the remaining time, XLDesired is in HH:MM format
					StartSpinningTime = Util->GetTimeSeconds() - (GetDesiredTimeInSeconds() - GetActualTimeInSeconds());	/// Time when it started is desired time less the remaining time
				}
				MachineStartTime = StartSpinningTime - 600;														/// Approximate how long the machine has been running as the spinning time minus 10 minutes
				w2t = GetW2tAsDouble();																								/// Initialize w2t to the latest value
	//--> Addlog note			Notes->AddNote("Machine running when the program was started.",7);
			}
			else{																																		/// If the machine is not spinning on startup, set the rotor speed to zero
				RequestedRotorSpeed="0";     																					/// Set the desired rotor speed to the current speed
				w2t = 0;																															/// Initialize w2t to 0
				CheckDesiredSettings();    																						/// Give the machine the desired settings
			}
			MachineStatus->UpdateDisplay();
			if(GetMachineStatus() == "Running"){     															 	/// See if the machine had been started prior to program initialization
				bMachineStarted=true;																									/// The machine is running
			}
			UpdateTimer->Enabled = true;
			bInitializingProgram = false;             															/// Set false inidicating communications with XL established, steppers are initialized
			UpdateStartStop();               																				/// Update the status of the 'Start' and 'Stop' buttons
			ConnectionStatus = Port.UpperCase()+" XL communications OK";
		}
	LastUpdateTime=(double)Util->GetTimeSeconds();
}
//----------------------------------------------------------------------------
void __fastcall TMachine::UpdateMachineStatus()
{/// Sends REQUESTSETTINGS & REQUESTSTATUS at proper time
	static double CheckSettingsTimer = (double)Util->GetTimeSeconds();			/// This is used to determine when to call REQUESTSETTINGS
	double Now=(double)Util->GetTimeSeconds();                              /// Get the present time... used to test to see if it is time to trigger various periodic events
	if (XLDesired->Changed
		|| (Now-CheckSettingsTimer >= 5*(UpdateTimer->Interval/1000))){				/// Only ask for the settings every 5 seconds+ the update timer period, and only ask for the settings (no status check). This avoids conflicts at the XLI.
		SendXLMessage(REQUESTSETTINGS);                            						/// Request the current machine settings if change has occurred
		CheckSettingsTimer=(double)Util->GetTimeSeconds();
	}
	else{																																		/// On those passes where there is no settings request, get the status
		SendXLMessage(REQUESTSTATUS);                              						/// Request the current machine status
	}
	AnsiString s = GetMachineStatus();                    									/// Change the update interval depending on the current machine status
	if(s == "Accelerating" || s == "Decelerating"){													/// Speed up the update rate
		UpdateTimer->Interval = 1000;
	}
	else{
		UpdateTimer->Interval = 2500;
	}
}
//---------------------q------------------------------------------------------
void __fastcall TMachine::UpdateRotorSpeed()
{/// Update the current rotor speed, log any milestones
	if(bSimulation){																												/// Determine whether we can start spinning & simulate new rotor speed
  	Simulator->SimulateNewRpm();																					/// Will only simulate it if it can
	}
	if(GetActualRotorSpeed() == 0 && bRotorSpinning){																		/// Rotor has come to a stop after spinning- log it
		//-->			RunLog->AddLog(23);
		bRotorSpinning=false;
	}
	if(GetActualRotorSpeed() >0){																												/// Note logic- if the rotor is spinning, the machine clearly has been started (whether or not programatically)
		bRotorSpinning=true;
	}
	else{																																		///Note logic- if the rotor is not spinning, the machine may still have been started (e.g. to pump down)
		bRotorSpinning=false;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMachine::UpdateZeroingSpeed()
{/// Check to see if the machine has reached 0 rpm
	if(bZeroingSpeed                                      /// Rotor slowing before start of experiment
		&& (GetActualRotorSpeed() == 0)){																/// Handle the case where the machine has reached 0 rpm
			bRotorSpinning=false;                    					/// Make sure that the system knows that the rotor is not spinning.
			bZeroingSpeed=false;                              /// Rpm reached zero when the rotor was being stopped prior to starting a method
	}
}
//----------------------------------------------------------------------------
void __fastcall TMachine::UpdateStartStop()
{/// Update the bCanStart and bCanStop flags
	if((!bMachineStarted                                        								/// The machine has not been started
		|| bStopping)                                              								/// or the machine is stopping
		&& CommunicationsEstablished()){                                 					/// And we are talking to the machine
		bCanStart = true;                         																/// Can start the machine
	}
	else{
		bCanStart = false;
	}
	if((bRotorSpinning || bMachineStarted)                                      /// The machine is spinning or the machine has been started
		&& CommunicationsEstablished()                             								/// and the communications has been established
		&& !bStopping){																														/// and the machine is not already stopping
		bCanStop = true;
	}
	else{
	 bCanStop = false;                                                          /// Machine is started, so can stop it
	}
}
//----------------------------------------------------------------------------
bool TMachine::CanSetValues()
{/// Returns true if the speed, temperature and time may be set
	bool ret = true;
	if(bInitializingProgram
		|| !CommunicationsEstablished()){
		ret = false;
	}
	return ret;
}
//----------------------------------------------------------------------------
#pragma package(smart_init)
